/***************************************************************************//**
 * @file    SoftTimer.cpp
 * @brief   SoftTimer - Arduino library for simple timers.
 *
 * @description
 * Provide a simple, quick to use timer object.
 * No hardware timer or interrupt is used.
 * Less code to type, and shorter programs because of not having to pepper
 * comparisons with millis() everywhere.
 *
 * Developed with Arduino IDE 1.6.10. May not run on Arduino 0.X.
 *
 * @TODO
 *
 * @author Volker Kuhlmann  @n
 * http://volker.top.geek.nz/contact.html
 *
 * @download
 * http://volker.top.geek.nz/arduino/
 *
 * @copyright
 * Copyright (C) 2016 by Volker Kuhlmann  @n
 *               http://volker.top.geek.nz/contact.html  @n
 * All rights reserved.
 * Released under the terms of the GNU General Public License (GPL) Version 3
 * or (at your option) any later version.  @n
 * See https://www.gnu.org/licenses/ for details.
 *
 * @history
 * - v1.0, 24 Aug 2016  Volker Kuhlmann
 *   - Initial version. Based on code in a project from 19 Aug 2016.
 * - v1.1, 25 Aug 2016  Volker Kuhlmann
 *   - Add missing intervalS(). Change unsigned short to explicit uint8_t.
 *   - Add example.
 * - v1.2, 05 Sep 2016  Volker Kuhlmann
 *   - Rename from SimpleTimer to SoftTimer.
 *   - Rename startS/intervalS to startN/intervalN, for Narrow / Wide.
 *   - Add SoftTimerExt.
 * - v1.3, 09 Sep 2016  Volker Kuhlmann
 *   - Add interval().
 * - v1.4, 13 Sep 2016  Volker Kuhlmann
 *   - Doxygen comments.
 *******************************************************************************
 */
// VERSION_DIST 1.4 	// Distribution version, for automatic extraction.

// Arduino environment
#if ARDUINO >= 100
#include <Arduino.h>
#else
#include <WProgram.h>
#endif


#include "SoftTimer.h"


/*!
 * Compares with the current value of millis().
 *
 * @note The Arduino millisecond counter overflows after 49.7 days.
 * This function fails if the timer's start and finish times cross the
 * overflow.
 *
 * @return bool True if timer has expired.
 * Return true for as long as the timer is in the expired state.
 */
bool SoftTimer::is_expired()
{
  return millis() >= _finish_ms;
}

/*!
 * Start the timer with a 16 bit (65535 ms max) delay from now.
 *
 * @param delay_ms uint16_t Timer delay in milliseconds.
 * @return void
 */
void SoftTimer::start(uint16_t delay_ms)
{
  _finish_ms = millis() + delay_ms;
}

/*!
 * Start the timer with a 32 bit (wide) delay from now.
 *
 * Uses unsigned long instead of uint32_t because that is what millis() uses.
 *
 * @param delay_ms unsigned long Timer delay in milliseconds.
 * @return void
 *
 */
void SoftTimer::startW(unsigned long delay_ms)
{
  _finish_ms = millis() + delay_ms;
}

/*!
 * Start the timer with an 8 bit (narrow, 255 ms max) delay from now.
 *
 * @param delay_ms uint8_t Timer delay in milliseconds.
 * @return void
 */
void SoftTimer::startN(uint8_t delay_ms)
{
  _finish_ms = millis() + delay_ms;
}

/*!
 * is_expired() and start() combined.
 * Because the timer is restarted, true is returned only once (unless the timer
 * has again expired before the next call).
 * @code
 *    SoftTimer timer;
 *    if (timer.interval(1000)) {
 *      // Do something once per second
 *    }
 * @endcode
 *
 * @param delay_ms uint16_t Timer delay in milliseconds.
 * @return bool True if timer has expired.
 */
bool SoftTimer::interval(uint16_t delay_ms)
{
  bool exp = is_expired();
  if (exp)
    start(delay_ms);
  return exp;
}

/*!
 * is_expired() and startW() combined.
 * See interval().
 *
 * Uses unsigned long instead of uint32_t because that is what millis() uses.
 *
 * @param delay_ms unsigned long Timer delay in milliseconds.
 * @return bool True if timer has expired.
 *
 */
bool SoftTimer::intervalW(unsigned long delay_ms)
{
  bool exp = is_expired();
  if (exp)
    startW(delay_ms);
  return exp;
}

/*!
 * is_expired() and startN() combined.
 * See interval().
 *
 * @param delay_ms uint8_t Timer delay in milliseconds.
 * @return bool True if timer has expired.
 *
 */
bool SoftTimer::intervalN(uint8_t delay_ms)
{
  bool exp = is_expired();
  if (exp)
    startN(delay_ms);
  return exp;
}


/*!
 * Empty constructor, but required.
 */
SoftTimerExt::SoftTimerExt()
{
}

/*!
 * Constructor that stores the timer delay (in milliseconds).
 * The timer is not started.
 *
 * @param delay_ms unsigned long Timer delay in milliseconds.
 */
SoftTimerExt::SoftTimerExt(unsigned long delay_ms)
{
  startW(delay_ms);
}

/*!
 * This function only works correctly if the timer interval has been stored
 * before, by calling start(), startW(), or startN() at least once before, or
 * using the constructor with delay parameter.
 * The start-functions can be called any time to change the timer delay for
 * subsequent calls to restart().
 *
 * @return void
 */
void SoftTimerExt::restart()
{
  startW(_length_ms);
}

/*!
 * Check if the timer has expired, and restart it if it has.
 * Similar to the SoftTimer interval functions, but start(), startW(), or
 * startN() must have been called at least once before, as for restart().
 *
 * @return bool True if timer has expired.
 */
bool SoftTimerExt::interval()
{
  return intervalW(_length_ms);
}

/*!
 * @return uint32_t Number of milliseconds before the timer expires.
 */
uint32_t SoftTimerExt::remaining()
{
  return _finish_ms - millis();
}

/*!
 * This returns ceil() of the remaining seconds.
 * I.e. 0s returns 0, 0.001 .. 1.000s returns 1, 1.001s .. 2.000s returns 2.
 *
 * Uses integer division.
 *
 * @return uint32_t Number of seconds before the timer expires.
 */
uint32_t SoftTimerExt::remaining_s()
{
  return (remaining() + 999) / 1000;
}

/*!
 * @return uint32_t Number of milliseconds since the timer was started.
 */
uint32_t SoftTimerExt::elapsed()
{
  return millis() - (_finish_ms - _length_ms);
}

/*!
 * Uses integer division.
 *
 * @return uint32_t Number of seconds since the timer was started.
 */
uint32_t SoftTimerExt::elapsed_s()
{
  return elapsed() / 1000;
}

/*!
 * Returns true once for each second that has elapsed since the timer was started.
 * This can be used e.g. to show elapsed time on a display, updating every second.
 *
 * @return bool True if another second of the timer delay has elapsed.
 */
bool SoftTimerExt::is_new_s()
{
  uint8_t this_s = remaining_s();
  if (this_s != _last_s) {
    _last_s = this_s;
    return true;
  }
  else
    return false;
}

/*!
 * Re-implementation, to also store the timer delay for use later.
 *
 * @param delay_ms uint16_t Timer delay in milliseconds.
 * @return void
 */
void SoftTimerExt::start(uint16_t delay_ms)
{
  // Start timer before saving delay value.
  SoftTimer::start(delay_ms);
  _length_ms = delay_ms;
}

/*!
 * Re-implementation, to also store the timer delay for use later.
 *
 * @param delay_ms unsigned long Timer delay in milliseconds.
 * @return void
 */
void SoftTimerExt::startW(unsigned long delay_ms)
{
  SoftTimer::startW(delay_ms);
  _length_ms = delay_ms;
}

/*!
 * Re-implementation, to also store the timer delay for use later.
 *
 * @param delay_ms uint8_t Timer delay in milliseconds.
 * @return void
 */
void SoftTimerExt::startN(uint8_t delay_ms)
{
  SoftTimer::startN(delay_ms);
  _length_ms = delay_ms;
}

/******************************************************************************/
