/***************************************************************************//**
 * @file    SoftTimer.h
 * @brief   SoftTimer - Arduino library for simple software timers.
 *
 * @description
 * Provide a simple, quick to use timer object.
 * No hardware timer or interrupt is used.
 * Less code to type, and shorter programs because of not having to pepper
 * comparisons with millis() everywhere.
 *
 * Developed with Arduino IDE 1.6.10. May not run on Arduino 0.X.
 *
 * @TODO
 *
 * @author Volker Kuhlmann  @n
 * http://volker.top.geek.nz/contact.html
 *
 * @download
 * http://volker.top.geek.nz/arduino/
 *
 * @copyright
 * Copyright (C) 2016 by Volker Kuhlmann  @n
 *               http://volker.top.geek.nz/contact.html  @n
 * All rights reserved.
 * Released under the terms of the GNU General Public License (GPL) Version 3
 * or (at your option) any later version.  @n
 * See https://www.gnu.org/licenses/ for details.
 *
 * @history
 * - v1.0, 24 Aug 2016  Volker Kuhlmann
 *   - Initial version. Based on code in a project from 19 Aug 2016.
 * - v1.1, 25 Aug 2016  Volker Kuhlmann
 *   - Add missing intervalS(). Change unsigned short to explicit uint8_t.
 *   - Add example.
 * - v1.2, 05 Sep 2016  Volker Kuhlmann
 *   - Rename from SimpleTimer to SoftTimer.
 *   - Rename startS/intervalS to startN/intervalN, for Narrow / Wide.
 *   - Add SoftTimerExt.
 * - v1.3, 09 Sep 2016  Volker Kuhlmann
 *   - Add interval().
 * - v1.4, 13 Sep 2016  Volker Kuhlmann
 *   - Doxygen comments.
 *******************************************************************************
 */

#ifndef SOFTTIMER_H_
#define SOFTTIMER_H_


/*!
 * @brief Software timer object.
 *
 * Simple timer that does not use interrupts.
 * It's easier to use and produces shorter code than using 4-byte comparisons
 * with millis() everywhere.
 */
class SoftTimer
{
  public:
    //! Return true if the timer has expired.
    bool is_expired();

    //! Start timer with up to 65535 ms.
    void start(uint16_t delay_ms);
    //! Start timer.
    void startW(unsigned long delay_ms);
    //! Start timer with up to 255 ms.
    void startN(uint8_t delay_ms);

    //! Return true when timer expires, and restart. 65535 ms max.
    bool interval(uint16_t delay_ms);
    //! Return true when timer expires, and restart.
    bool intervalW(unsigned long delay_ms);
    //! Return true when timer expires, and restart. 255 ms max.
    bool intervalN(uint8_t delay_ms);

  protected:
  	//! Expiry time, in milliseconds.
    unsigned long _finish_ms;
};


/*!
 * @brief Software timer object with extended features.
 *
 * The timer can be restarted without having to give the interval again.
 * Elapsed and remaining times are availble.
 * Uses more memory.
 */
class SoftTimerExt
  : public SoftTimer
{
  public:
  	//! Default constructor, does nothing.
    SoftTimerExt();
    //! Save the timer interval without starting timer.
    SoftTimerExt(unsigned long delay_ms);

    //! Restart timer (that was previously started).
    void restart();

	  //! Return true when timer expires, and restart.
    bool interval();

    //! Return remaining timer milliseconds.
    uint32_t remaining();
    //! Return remaining timer seconds.
    uint32_t remaining_s();
    //! Return elapsed milliseconds since (re-)start.
    uint32_t elapsed();
    //! Return elapsed seconds since (re-)start.
    uint32_t elapsed_s();

    //! Return true if a new second has started since last call.
    bool is_new_s();

    //! Same as for SoftTimer.
    void start(uint16_t delay_ms);
    //! Same as for SoftTimer.
    void startW(unsigned long delay_ms);
    //! Same as for SoftTimer.
    void startN(uint8_t delay_ms);

  protected:
  	//! Timer interval in milliseconds.
    unsigned long _length_ms;

  private:
  	//! Hold the seconds when is_new_s() was last called.
    uint8_t _last_s;
};


#endif // SOFTTIMER_H_

/******************************************************************************/
